from typing import Union, Dict, Any,Optional,List
from pydantic import BaseModel
from wetro.custom_types import CollectionID,ResourceID, JSONSchema, CollectionItem, InsertStatus

class CreateCollectionResponse(BaseModel):
    collection_id: CollectionID
    success: bool

class GetCollectionResponse(BaseModel):
    success: bool
    found: bool
    collection_id: Optional[str] = None

class ListCollectionResponse(BaseModel):
    count: Optional[int] = None
    next: Optional[str] = None
    previous: Optional[str] = None
    results: List[CollectionItem]

class QueryResponse(BaseModel):
    response: Union[str, JSONSchema]
    tokens: int
    success: bool

class ChatResponse(BaseModel):
    response: Union[str, JSONSchema]
    tokens: int
    success: bool

class InsertResponse(BaseModel):
    resource_id : ResourceID
    success: bool
    tokens: Optional[int] = None
    status: Optional[InsertStatus] = None

class CheckInsertResponse(BaseModel):
    resource_id : ResourceID
    success: bool
    status: Optional[InsertStatus] = None

class RemoveResponse(BaseModel):
    success: bool

class DeleteCollectionResponse(BaseModel):
    message: str
    success: bool

class CategorizeResponse(BaseModel):
    response: JSONSchema
    tokens: int
    success: bool

class GenerateTextResponse(BaseModel):
    response: Union[str, JSONSchema]
    tokens: int
    success: bool

class ImageToTextResponse(BaseModel):
    response: str
    tokens: int
    success: bool

class ExtractDataResponse(BaseModel):
    response: JSONSchema
    tokens: int
    success: bool

class MarkdownCoverterResponse(BaseModel):
    response: str
    tokens: int
    success: bool


class TrascribeResponse(BaseModel):
    response: JSONSchema
    tokens: int
    success: bool

class WetrocloudAPIResponse:
    def __init__(self, data: Dict[str, Any]):
        for key, value in data.items():
            setattr(self, key, value)

    def to_dict(self) -> Dict[str, Any]:
        return self.__dict__



from wetro import WetroRAG, WetroTools, Wetrocloud
import os

def main():
    pass
    # Test RAG operations via SDK
    print("=== Testing RAG Operations via SDK ===")
    # wetro_client = Wetrocloud(api_key="c80d5cb1f295297ef77eb82f42aafe09b71625e1",base_url="http://127.0.0.1:8000")
    wetro_client = Wetrocloud(api_key="wtc-sk-8f956b955c76a1049aa87b11e5f589104baf7e73")
    # wetro_client.collection.get_or_create_collection_id("test_collection_sdk")
    # collection_resp = wetro_client.collection.create_collection("test__sdk040425")
    # print("SDK Create Collection Response: %s", collection_resp.model_dump())
    # all_collection_resp = wetro_client.collection.get_collection_list()
    # print("SDK Get All Collection Response: %s", all_collection_resp.model_dump())
    
    # insert_resp = wetro_client.collection.insert_resource(resource="https://medium.com/@wetrocloud/are-image-models-getting-out-of-hand-068b13090556", type="web", collection_id="unique_collection_id_1")
    # print("SDK Insert Response: %s", insert_resp.model_dump())
    # print(os.path.abspath("test-resources/1.pdf"))
   
    # insert_resp = wetro_client.collection.insert_resource(resource="./tests/test-resources/1.pdf", type="file", collection_id="unique_collection_id_1")
    # print("SDK Insert Response: %s", insert_resp.model_dump())
    
    # Test basic query
    # query_resp = wetro_client.collection.query_collection(request_query="What is the collection about?", collection_id="test__sdk040425")
    # print("SDK Query Response: %s", query_resp.model_dump())
    
    # Test streaming query
    # print("SDK Streaming Query Responses:")
    # stream_resp = wetro_client.collection.query_collection(request_query="Streaming test", stream=True, collection_id="test_collection_sdk")
    # for chunk in stream_resp:
    #     print(chunk.model_dump())
    
    # # Test structured query
    # structured_query = wetro_client.collection.query_collection(
    #     request_query="What is this about?",
    #     json_schema={"title": "string", "summary": "string"},
    #     json_schema_rules=["Summaries the collection"],
    #     collection_id="test_collection_sdk"
    # )
    # print("SDK Structured Query Response: %s", structured_query.model_dump())
    
    # chat_history= [{"role": "user", "content": "What is this all about?"}]
    # chat_resp = wetro_client.collection.chat(message="What is this all about?", chat_history=chat_history, collection_id="test_collection_sdk")
    # print("SDK Chat Response: %s", chat_resp.model_dump())

    # delete_resource_resp = wetro_client.collection.delete_resource(insert_resp.resource_id, collection_id="test_collection_sdk")
    # print("SDK Delete Resource Response: %s", delete_resource_resp.model_dump())
    
    # delete_resp = wetro_client.collection.delete_collection(collection_id="test_collection_sdk")
    # print("SDK Delete Response: %s", delete_resp.model_dump())
    
    # # Test Tools operations via SDK
    # print("=== Testing Tools Operations via SDK ===")

    # categorize_resp = wetro_client.categorize(
    #     resource="Match review: Example vs. Test.",
    #     type="text",
    #     json_schema={"label": "string"},
    #     categories=["sports", "entertainment"],
    #     prompt="Categorize the text to see which category it best fits"
    # )
    # print("SDK Categorize Response: %s", categorize_resp.model_dump())
    
    # generate_resp = wetro_client.generate_text(
    #     messages=[{"role": "user", "content": "What is a large language model?"}],
    #     model="gpt-4"
    # )
    # print("SDK Generate Text Response: %s", generate_resp.model_dump())
    
    # ocr_resp = wetro_client.image_to_text(
    #     image_url="https://images.squarespace-cdn.com/content/v1/54822a56e4b0b30bd821480c/45ed8ecf-0bb2-4e34-8fcf-624db47c43c8/Golden+Retrievers+dans+pet+care.jpeg",
    #     request_query="What animal is in this image?"
    # )
    # print("SDK Image to Text Response: %s", ocr_resp.model_dump())
    
    # extract_resp = wetro_client.extract(
    #     website="https://medium.com/@wetrocloud/are-image-models-getting-out-of-hand-068b13090556",
    #     json_schema={"title" : "<string>", "models" : "<string>"}
    # )
    # print("SDK Extract Data Response: %s", extract_resp.model_dump())

    # markdown_response = wetro_client.markdown_converter(
    #     link="https://medium.com/@wetrocloud/are-image-models-getting-out-of-hand-068b13090556",
    #     resource_type="web"
    # )
    # print(markdown_response)
    # markdown_response = wetro_client.markdown_converter(
    #     link="https://res.cloudinary.com/dfcunvtqz/image/upload/v1746792329/handwritten_note_3_mzx4sc.png",
    #     resource_type="image"
    # )
    # print(markdown_response)

    markdown_response = wetro_client.markdown_converter(
        link="./tests/test-resources/1.pdf",
        resource_type="file"
    )
    print(markdown_response)

    # transcript_response = wetro_client.transcript(
    #     link="https://www.youtube.com/watch?v=4c9_zZJlZRw&ab_channel=TayoAina",
    #     resource_type="youtube"
    # )
    # print(transcript_response)

if __name__ == "__main__":
    main()



import unittest
from wetro import Wetrocloud
import time
import os

# To run the tests:
# $ python -m unittest test_wetrocloud.py

# To run only one test
# $ python -m unittest test_wetrocloud.TestWetrocloudAPI.test_get_all_collections

class TestWetrocloudAPI(unittest.TestCase):

    def setUp(self):
        self.collection_id = "my_unique_collection_id"
        self.resource_id = "my_unique_resource_id"
        # Initialize the Wetrocloud client with your real API key
        self.client = Wetrocloud(api_key="wtc-sk-8f956b955c76a1049aa87b11e5f589104baf7e73")

    # Create collection
    def test_create_collection(self):
        # Test the create_collection method
        collection_id = "my_unique_collection_id_12345678"

        # Create a new collection
        response = self.client.collection.create_collection(collection_id=collection_id)
        self.collection_id = collection_id  # Store the collection ID for later use
        print("Create Collection Response:", response)

        self.assertEqual(response.collection_id, collection_id)


    # def test_create_or_get_collection(self):
    #     # Test the get_or_create_collection_id method (creating or fetching a collection)
    #     collection_id = "my_unique_collection_id"

    #     # Create or get collection
    #     response = self.client.collection.get_or_create_collection_id(collection_id=collection_id)
    #     print("Get or Create Collection Response:", response)

    #     self.assertEqual(response.collection_id, collection_id)

    # Get all collections
    def test_get_all_collections(self):
        # Test getting all collections
        response = self.client.collection.get_collection_list()
        print("Get All Collections Response:", response)
        
        self.assertIsNotNone(response.count)

    # Insert resource
    def test_insert_resource(self):
        # Test inserting a web resource into the collection
        collection_id = self.collection_id
        resource_url = "https://medium.com/@wetrocloud/are-image-models-getting-out-of-hand-068b13090556"
        
        # Insert the resource
        response = self.client.collection.insert_resource(collection_id=collection_id, resource=resource_url, type="web")
        print("Insert Resource Response:", response)

        self.resource_id= response.resource_id  # Store the resource ID for later use   
        
        self.assertIsNotNone(response.resource_id)
        self.assertEqual(response.success, True)

        # Upload Insert resource
    def test_upload_insert_resource(self):
        # Test inserting a web resource into the collection
        collection_id = self.collection_id
        resource_url = "./tests/test-resources/1.pdf"
        
        # Insert the resource
        response = self.client.collection.insert_resource(collection_id=collection_id, resource=resource_url, type="file")
        print("Insert Resource Response:", response)

        self.resource_id= response.resource_id  # Store the resource ID for later use   
        
        self.assertIsNotNone(response.resource_id)
        self.assertEqual(response.success, True)

    # Query collection
    def test_query_collection(self):
        # Test querying a collection
        collection_id = self.collection_id
        request_query = "What are the key points of the article?"
        
        # Query the collection
        response = self.client.collection.query_collection(collection_id=collection_id, request_query=request_query)
        print("Query Collection Response:", response)
        
        self.assertEqual(response.success, True)
        self.assertIsNotNone(response.response)

    # Query collection with JSON schema
    def test_query_collection_with_schema_and_rules(self):
        # Define the JSON schema and rules
        json_schema = [{"point_number": "<int>", "point": "<str>"}]
        rules = ["Only 5 points", "Strictly return JSON only"]

        # Query the collection with schema and rules
        response = self.client.collection.query_collection(
            collection_id=self.collection_id,
            request_query="What are the key points of the article?",
            json_schema=json_schema,
            json_schema_rules=rules
        )
        
        print("Query Collection Response:", response)

        self.assertEqual(response.success, True)
        self.assertIsNotNone(response.response)


    def test_streaming_response(self):
        # Query the collection with the stream flag set to True
        streaming_response = self.client.collection.query_collection(
            collection_id=self.collection_id,
            request_query="Give me a detailed summary of the article",
            stream=True
        )

        # Initialize a variable to store the complete response
        full_response = ""

        # Process the streaming response
        for chunk in streaming_response:
            # Check that the chunk has a 'response' attribute and that it's a string
            self.assertTrue(hasattr(chunk, 'response'))
            self.assertIsInstance(chunk.response, str)

            # Print or accumulate the chunk in the full response
            full_response += chunk.response

        # Ensure the full response is not empty (to confirm data was streamed)
        self.assertGreater(len(full_response), 0)

        # Optionally, print the full response for review
        print("Full streamed response:", full_response)


    def test_chat_with_history(self):
        # Define the initial chat history
        chat_history = [
            {"role": "user", "content": "What is this collection about?"}, 
            {"role": "system", "content": "It stores research papers on AI technology."}
        ]

        # Define the new message to continue the conversation
        new_message = "Can you explain the latest paper's methodology?"

        # Continue the conversation with context
        chat_response = self.client.collection.chat(
            collection_id=self.collection_id,
            message=new_message,
            chat_history=chat_history
        )
        
        # Print the chat response
        print("Chat Response:", chat_response)

        # Assert the response has the expected structure
        self.assertEqual(chat_response.success, True)
        self.assertIsNotNone(chat_response.response)

    # Delete resource
    def test_delete_resource(self):
        # Test querying a collection
        collection_id = self.collection_id
        resource_id = self.resource_id

        # Delete the resource
        response = self.client.collection.delete_resource(resource_id=resource_id, collection_id=collection_id)
        print("Delete Resource Response:", response)

        self.assertEqual(response.success, True)

    # Delete collection
    def test_delete_collection(self):
        # Test deleting a collection
        collection_id = self.collection_id

        # Delete the collection
        response = self.client.collection.delete_collection(collection_id=collection_id)
        print("Delete Collection Response:", response)

        self.assertEqual(response.success, True)



    def test_text_generation(self):
        # Define the input messages for the text generation
        messages = [
            {"role": "system", "content": "You are a helpful assistant."}, 
            {"role": "user", "content": "Write a short poem about technology."}
        ]
        
        # Call the generate_text method to get the response from the specified model
        response = self.client.generate_text(
            messages=messages,
            model="gpt-4"  # Specify the model here
        )
        
        # Print the response for review
        print("Generated Text:", response)

        # Assert that the response contains a valid text
        self.assertEqual(response.success, True)
        self.assertIsNotNone(response.response)

    
    # Categorize data
    def test_data_categorization(self):
        # Define the resource (content to categorize)
        resource = "match review: John Cena vs. The Rock."
        
        # Define the json_schema for categorization
        json_schema = {"label": "string"}

        # Define the possible categories
        categories = ["wrestling", "entertainment", "sports", "news"]

        # Call the categorize method
        categorize_response = self.client.categorize(
            resource=resource,
            type="text",  # Specify that the resource type is text
            json_schema=json_schema,
            categories=categories,
            prompt="Categorize the text to see which category it best fits"
        )

        # Print the categorize response for review
        print("Categorize Response:", categorize_response)

        # Assert that the response contains the expected status
        self.assertEqual(categorize_response.success, True)
        self.assertIsNotNone(categorize_response.response)

    
    # Image to text
    def test_image_to_text(self):
        # Define the image URL and the query to ask about the image
        image_url = "https://images.squarespace-cdn.com/content/v1/54822a56e4b0b30bd821480c/45ed8ecf-0bb2-4e34-8fcf-624db47c43c8/Golden+Retrievers+dans+pet+care.jpeg"
        request_query = "What animal is in this image?"

        # Call the image_to_text method to extract text from the image and answer the query
        ocr_response = self.client.image_to_text(
            image_url=image_url,
            request_query=request_query
        )
        
        # Print the OCR response for review
        print("OCR Response:", ocr_response)

        # Assert that the response contains the expected status and answer
        self.assertEqual(ocr_response.success, True)
        self.assertIsNotNone(ocr_response.response)


    # Data extraction
    def test_data_extraction(self):
        # Define the website URL from which data will be extracted
        website_url = "https://medium.com/@wetrocloud/are-image-models-getting-out-of-hand-068b13090556"

        # Define the json_schema for the expected structured data
        json_schema ={"title" : "<string>", "models" : "<string>"}

        # Call the extract method to get structured data from the website
        extract_response = self.client.extract(
            website=website_url,
            json_schema=json_schema
        )
        
        # Print the extraction response for review
        print("Extract Response:", extract_response)

        # Assert that the response contains the expected status and answer
        self.assertEqual(extract_response.success, True)
        self.assertIsNotNone(extract_response.response)


    def tearDown(self):
        # Optional: Clean up after tests, like deleting the collection or resources.
        # Be careful with this as it will actually delete your collection.
        # self.client.collection.delete_collection(collection_id="my_unique_collection_id")
        pass


if __name__ == "__main__":
    unittest.main()




from wetro.api_client import WetrocloudClient
from wetro.rag import WetroRAG
from wetro.toolkit import WetroTools
from typing import List,Optional,Union, Any
from wetro.custom_types import ChatModel, ResourceType, Categories, JSONSchema, Message, URL, MarkdownResourceType, TranscribeResourceType
from wetro.custom_response import CategorizeResponse, GenerateTextResponse, ImageToTextResponse, ExtractDataResponse, MarkdownCoverterResponse, TrascribeResponse

class Wetrocloud:
    """
    Client that allows setting the API key only once.
    
    Usage:
        client = Wetrocloud(api_key="your_api_key")
        rag_client = client.rag
        tools_client = client.tools
    """
    def __init__(
            self, 
            api_key: str, 
            base_url: str = "https://api.wetrocloud.com", 
            timeout: int = 30
        ):
        self._client = WetrocloudClient(api_key, base_url, timeout)
        self.rag = WetroRAG(client=self._client)
        self.tools = WetroTools(client=self._client)
        self.collection = self.rag.collection

    def categorize(
            self, 
            resource: str, 
            type: ResourceType, 
            json_schema: JSONSchema, 
            categories: Categories,
            prompt: str
        ) -> CategorizeResponse:
        """
        Categories your resource.
        """
        return self.tools.categorize(resource, type, json_schema, categories, prompt)

    def generate_text(
            self, 
            messages: List[Message], 
            model: ChatModel
        ) -> GenerateTextResponse:
        """
        Generate Text-Response based on message
        """
        return self.tools.generate_text(messages, model)

    def image_to_text(
            self, 
            image_url: URL, 
            request_query: str
        ) -> ImageToTextResponse:
        """
        Generate Text-Response based on image
        """
        return self.tools.image_to_text(image_url, request_query)

    def extract(
            self, 
            website: URL, 
            json_schema: JSONSchema
        ) -> ExtractDataResponse:
        """
        Extract Data from Websites
        """
        return self.tools.extract(website, json_schema)
    
    def markdown_converter(
            self, 
            link: Union[str, bytes, Any], 
            resource_type : MarkdownResourceType
        ) -> MarkdownCoverterResponse:
        """
        Convert Data to Markdown
        """
        return self.tools.markdown_converter(link, resource_type )
    
    def transcript(
            self, 
            link: URL, 
            resource_type : TranscribeResourceType
        ) -> TrascribeResponse:
        """
        Trascribe Youtube Videos
        """
        return self.tools.transcript(link, resource_type )
    


import os
import json
import requests
from pathlib import Path
from typing import Any, Dict, List, Optional, Iterable, Union
from wetro.custom_types import (
    ChatModel, 
    ResourceType, 
    ChatHistory, 
    JSONSchema, 
    Categories, 
    CollectionID, 
    ResourceID, 
    URL, 
    Message,
    MarkdownResourceType,
    TranscribeResourceType
    )
from wetro.custom_response import (
    CreateCollectionResponse, 
    GetCollectionResponse,
    QueryResponse, 
    ChatResponse, 
    InsertResponse, 
    RemoveResponse, 
    DeleteCollectionResponse, 
    CategorizeResponse, 
    GenerateTextResponse, 
    ImageToTextResponse, 
    ExtractDataResponse,
    WetrocloudAPIResponse,
    ListCollectionResponse,
    CheckInsertResponse,
    MarkdownCoverterResponse,
    TrascribeResponse
    )

class WetrocloudAPIError(Exception):
    """Custom exception for Wetrocloud API errors."""
    def __init__(self, message, status_code=None, payload=None):
        super().__init__(message)
        self.status_code = status_code
        self.payload = payload

class WetrocloudClient:
    """
    A client for interacting with the Wetrocloud API.
    
    Attributes:
        api_key (str): Your API key for authenticating with the Wetrocloud API.
        base_url (str): The base URL for the Wetrocloud API endpoints.
        timeout (int): The timeout for HTTP requests (in seconds).
    """
    def __init__(
            self, 
            api_key: str, 
            base_url: str = "https://api.wetrocloud.com", 
            timeout: int = 30
        ):
        self.api_key = api_key
        self.base_url = base_url.rstrip("/")  # Ensure no trailing slash
        self.timeout = timeout
        self.headers = {
            "Authorization": f"Token {self.api_key}",
            "Content-Type": "application/json",
        }


    def _parse_error(self, response: requests.Response) -> str:
        """Helper to extract error messages from the response."""
        try:
            error_data = response.json()
        except ValueError:
            return response.text or "Unknown error"
        
        # Check for common error keys
        if "error" in error_data:
            return error_data["error"]
        if "detail" in error_data:
            return error_data["detail"]
        # For missing parameter errors, join messages together.
        if isinstance(error_data, dict):
            errors = []
            for field, messages in error_data.items():
                if isinstance(messages, list):
                    errors.append(f"{field}: {', '.join(messages)}")
                else:
                    errors.append(f"{field}: {messages}")
            return "; ".join(errors)
        return str(error_data)

    def _request(
            self, 
            method: str, 
            endpoint: str, 
            params: dict = None, 
            data: dict = None, 
            response_model: Optional[Any] = None
        ) -> Any:
        """
        Internal method to make HTTP requests to the API.
        Sends JSON data.
        
        Args:
            method (str): HTTP method (GET, POST, PUT, DELETE).
            endpoint (str): API endpoint (e.g., '/v1/create/').
            params (dict, optional): Query parameters.
            data (dict, optional): Request body.
        
        Returns:
            dict: Parsed JSON response.
        
        Raises:
            WetrocloudAPIError: If the request fails.
        """
        url = f"{self.base_url}{endpoint}"
        try:
            if params==None:
                params = {
                    "referrer": "PYTHON_SDK"
                }
            else:
                params["referrer"]= "PYTHON_SDK"
            response = requests.request(method, url, headers=self.headers, params=params, json=data, timeout=self.timeout)
            response.raise_for_status()
            json_response = response.json()
            if response_model is not None:
                return response_model.model_validate(json_response)
            return WetrocloudAPIResponse(json_response)
        except requests.exceptions.HTTPError as http_err:
            error_msg = self._parse_error(http_err.response)
            raise WetrocloudAPIError(f"HTTP error occurred: {error_msg}", status_code=http_err.response.status_code, payload=http_err.response.json())
        except requests.exceptions.RequestException as req_err:
            raise WetrocloudAPIError(f"Request exception: {req_err}")

    def _request_multipart(
            self, 
            method: str, 
            endpoint: str, 
            data: dict, 
            response_model: Optional[Any] = None
        ) -> WetrocloudAPIResponse:
        """
        Internal method to make multipart/form-data requests.
        Used for endpoints that require form data instead of JSON (e.g. chat and remove-resource).
        
        Args:
            method (str): HTTP method (POST or DELETE).
            endpoint (str): API endpoint.
            data (dict): Form data parameters.
        
        Returns:
            dict: Parsed JSON response.
        
        Raises:
            WetrocloudAPIError: If the request fails.
        """
        url = f"{self.base_url}{endpoint}"
        try:
            # Only the Authorization header is needed; let requests set the Content-Type automatically.
            headers = {"Authorization": self.headers["Authorization"]}
            # Prepare multipart form data; if a value is not a string, convert it to JSON.
            files = {k: (None, v if isinstance(v, str) else json.dumps(v)) for k, v in data.items()}
            response = requests.request(method, url, headers=headers, files=files, timeout=self.timeout)
            response.raise_for_status()
            json_response = response.json()
            if response_model is not None:
                return response_model.model_validate(json_response)
            return WetrocloudAPIResponse(json_response)
        except requests.exceptions.HTTPError as http_err:
            error_msg = self._parse_error(http_err.response)
            raise WetrocloudAPIError(f"HTTP error occurred: {error_msg}", status_code=http_err.response.status_code, payload=http_err.response.json())
        except requests.exceptions.RequestException as req_err:
            raise WetrocloudAPIError(f"Request exception: {req_err}")

    def _request_stream(
            self,
            method: str,
            endpoint: str,
            params: dict = None,
            data: dict = None,
            response_model: Optional[Any] = None
        ) -> Iterable[Any]:
        url = f"{self.base_url}{endpoint}"
        try:
            if params==None:
                params = {
                    "referrer": "PYTHON_SDK"
                }
            else:
                params["referrer"]= "PYTHON_SDK"
            response = requests.request(method, url, headers=self.headers, params=params, json=data, timeout=self.timeout, stream=True)
            response.raise_for_status()
            for line in response.iter_lines():
                if line:
                    chunk = json.loads(line)
                    if response_model is not None:
                        yield response_model.model_validate(chunk)
                    else:
                        yield chunk
        except requests.exceptions.HTTPError as http_err:
            error_msg = self._parse_error(http_err.response)
            raise WetrocloudAPIError(
                f"HTTP error occurred: {error_msg}",
                status_code=http_err.response.status_code,
                payload=http_err.response.json()
            )
        except requests.exceptions.RequestException as req_err:
            raise WetrocloudAPIError(f"Request exception: {req_err}")
    
    def _upload_file(self,collection_id: CollectionID, resource: Union[str, bytes, Any]) -> str:
        """
        Uploads a file to the file-upload service and returns the URL.
        If resource is a string that doesn't start with "http", it's assumed to be a file path.
        Otherwise, resource can be a bytes-like object or a file-like object.
        """
        upload_url = "https://file-upload-service-python.vercel.app/upload/"
        files = {}
        if isinstance(resource, str) and not resource.startswith("http"):
            # Assume resource is a file path.
            resource = Path(resource).expanduser().resolve().as_posix()
            # print(resource)
            if not os.path.exists(resource):
                raise ValueError(f"File {resource} does not exist.")
            files["file"] = open(resource, "rb")
        else:
            # If resource is already a file-like object or bytes.
            files["file"] = resource

        try:
            response = requests.post(upload_url,data={"collection_id":collection_id}, files=files)
            response.raise_for_status()
            json_response = response.json()
            file_url = json_response.get("url")
            if not file_url:
                raise ValueError("File upload did not return a URL.")
            return file_url
        finally:
            if isinstance(resource, str) and not resource.startswith("http"):
                files["file"].close()
    
    def _get_resource(self,collection_id: CollectionID, resource: Union[str, bytes, Any]) -> str:
        if not (isinstance(resource, str) and resource.startswith("http")):
            resource = self._upload_file(collection_id,resource)
        return resource
    
    # --- New functions for collection endpoints ---
    def create_collection(
            self,
            collection_id: CollectionID
        ) -> CreateCollectionResponse:
        """
            Create a new collection.

            Endpoint:
                POST /v1/collection/create/ (&#8203;:contentReference[oaicite:6]{index=6})
        """
        payload = {
            "collection_id": collection_id,
        }
        
        return self._request("POST", "/v1/collection/create/",data=payload, response_model=CreateCollectionResponse)
    
    def get_collection(self, collection_id: CollectionID) -> GetCollectionResponse:
        """
            Get a new collection.

            Endpoint:
                POST /v1/collection/get/<collection_id>/ (&#8203;:contentReference[oaicite:6]{index=6})
        """
        endpoint = f"/v1/collection/get/{collection_id}/"
        
        return self._request("GET", endpoint, response_model=GetCollectionResponse)
    
    def list_collection(self) -> ListCollectionResponse:
        """
            List all collections.

            Endpoint:
                POST /v1/collection/get/<collection_id>/ (&#8203;:contentReference[oaicite:6]{index=6})
        """
        endpoint = f"/v1/collection/all/"
        
        return self._request("GET", endpoint, response_model=ListCollectionResponse)

    def query_collection(
            self, 
            collection_id: CollectionID, 
            request_query: str, 
            model: Optional[ChatModel] = None,
            json_schema: Optional[JSONSchema] = None,
            json_schema_rules: Optional[List[str]] = None,
            stream: bool = False
        ) -> Union[QueryResponse, Iterable[QueryResponse]]:
        """
        Query a collection for answers.
        
        Args:
            collection_id (str): The collection ID.
            request_query (str): The query string.
            model (str, optional): Optional model identifier.
        
        Returns:
            dict: Response including the query answer, tokens, and success flag.
        
        Endpoint:
            POST /v1/query/ (&#8203;:contentReference[oaicite:8]{index=8})
        """
        payload = {
            "collection_id": collection_id,
            "request_query": request_query
        }
        if model:
            payload["model"] = model
        # Enforce that both json_schema and json_schema_rules are provided together.
        # Validate structured output parameters
        if (json_schema is not None or json_schema_rules is not None) and stream:
            raise ValueError("Streaming mode does not support json_schema and json_schema_rules.")
        if (json_schema is not None and json_schema_rules is None) or (json_schema_rules is not None and json_schema is None):
            raise ValueError("Both json_schema and json_schema_rules must be provided together, or neither.")
        if json_schema is not None and json_schema_rules is not None:
            payload["json_schema"] = json_schema
            payload["json_schema_rules"] = json_schema_rules
        if stream:
            payload["stream"] = True
            return self._request_stream("POST", "/v1/collection/query/", data=payload, response_model=QueryResponse)
        else:
            return self._request("POST", "/v1/collection/query/", data=payload, response_model=QueryResponse)

    def chat_with_collection(
            self, 
            collection_id: CollectionID, 
            message: str, 
            chat_history: ChatHistory = [], 
            model: Optional[ChatModel] = None,
            stream: bool = False  
        ) -> Union[QueryResponse, Iterable[ChatResponse]]:
        """
        Chat with a collection to have a conversation based on its resources.
        
        Args:
            collection_id (str): The collection ID.
            message (str): The current message.
            chat_history (List[dict]): A list of previous chat messages (each a dict with 'role' and 'content').
        
        Returns:
            dict: Response including the conversational reply, tokens, and success flag.
        
        Endpoint:
            POST /v1/collection/chat/ (&#8203;:contentReference[oaicite:9]{index=9})
        """
        form_data = {
            "collection_id": collection_id,
            "message": message,
            "chat_history": chat_history
        }
        if model:
            form_data["model"] = model
        
        if stream:
            form_data["stream"] = True
            return self._request_stream("POST", "/v1/collection/chat/", data=form_data, response_model=ChatResponse)
        else:
            return self._request("POST", "/v1/collection/chat/", data=form_data, response_model=ChatResponse)

    def insert_resource(
            self, 
            collection_id: CollectionID, 
            resource: Union[str, bytes, Any], 
            type: ResourceType
        ) -> InsertResponse:
        """
            Insert a resource into a collection.
            
            Args:
                collection_id (str): The ID of the collection.
                resource (str): The resource URL, text, file URL, etc.
                type (str): The type of resource (e.g., 'web', 'file', 'text', 'json', 'youtube').
            
            Returns:
                dict: Response including success flag and tokens used.
            
            Endpoint:
                POST /v1/insert/ (&#8203;:contentReference[oaicite:7]{index=7})
        """
        resource = self._get_resource(collection_id,resource)
        payload = {
            "collection_id": collection_id,
            "resource": resource,
            "type": type
        }
        
        return self._request("POST", "/v1/resource/insert/", data=payload, response_model=InsertResponse)
    
    def check_insert_resource(
            self, 
            collection_id: CollectionID, 
            resource_id: ResourceID
        ) -> CheckInsertResponse:
        """
            Check Status of Inserted resource in a collection.
            
            Args:
                collection_id (str): The ID of the collection.
                resource_id (str): The resource ID to check.
            
            Returns:
                dict: Response including success flag and tokens used.
            
            Endpoint:
                POST /v1/insert/check/ (&#8203;:contentReference[oaicite:7]{index=7})
        """
        payload = {
            "collection_id": collection_id,
            "resource_id": resource_id
        }
        
        return self._request("POST", "/v1/resource/insert/check/", data=payload, response_model=CheckInsertResponse)
    
    def remove_resource(
            self,
            collection_id: CollectionID, 
            resource_id: ResourceID
        ) -> RemoveResponse:
        """
        Remove a resource from a collection.
        
        Args:
            collection_id (str): The collection ID.
            resource_id (str): The resource ID to remove.
        
        Returns:
            dict: Response indicating whether the removal was successful.
        
        Endpoint:
            DELETE /v1/remove/resource/ (&#8203;:contentReference[oaicite:10]{index=10})
        """
        form_data = {
            "collection_id": collection_id,
            "resource_id": resource_id
        }
        
        return self._request("DELETE", "/v1/resource/remove/", data=form_data, response_model=RemoveResponse)

    def delete_collection(
            self, 
            collection_id: CollectionID
        ) -> DeleteCollectionResponse:
        """
        Delete a collection.
        
        Args:
            collection_id (str): The ID of the collection to delete.
        
        Returns:
            dict: Response including a message and success flag.
        
        Endpoint:
            DELETE /v1/delete/ (&#8203;:contentReference[oaicite:11]{index=11})
        """
        payload = {
            "collection_id": collection_id
        }
        return self._request("DELETE", "/v1/collection/delete/", data=payload, response_model=DeleteCollectionResponse)
    
     # New functions for additional endpoints

    def categorize_data(
            self, 
            resource: str, 
            type: ResourceType, 
            json_schema: JSONSchema, 
            categories: Categories,
            prompt:str
        ) -> CategorizeResponse:
        """
        Categorize a resource using a JSON schema and rules.
        
        Endpoint: POST /v1/categorize/
        """
        payload = {
            "resource": resource,
            "type": type,
            "json_schema": json_schema,
            "categories": categories,
            "prompt": prompt
        }
        return self._request("POST", "/v1/categorize/", data=payload, response_model=CategorizeResponse)

    def generate_text(
            self, 
            messages: List[Message], 
            model: ChatModel
        ) -> GenerateTextResponse:
        """
        Generate text using the provided conversation messages and model.
        
        Endpoint: POST /v1/text-generation/
        Note: This endpoint expects multipart/form-data.
        """
        form_data = {
            "messages": messages,  # Can be a list of message objects
            "model": model
        }
        return self._request("POST", "/v1/text-generation/", data=form_data, response_model=GenerateTextResponse)

    def image_to_text(
            self, 
            image_url: URL, 
            request_query: str
        ) -> ImageToTextResponse:
        """
        Convert an image to text (OCR) and optionally answer a query about the image.
        
        Endpoint: POST /v1/image-to-text/
        """
        payload = {
            "image_url": image_url,
            "request_query": request_query
            }
        return self._request("POST", "/v1/image-to-text/", data=payload, response_model=ImageToTextResponse)

    def extract_data(
            self, 
            website: URL, 
            json_schema: JSONSchema
        ) -> ExtractDataResponse:
        """
        Extract structured data from a website using the provided JSON schema.
        
        Endpoint: POST /v1/data-extraction/
        Note: This endpoint expects multipart/form-data.
        """
        form_data = {
            "website": website,
            "json_schema": json_schema
        }
        return self._request("POST", "/v1/data-extraction/", data=form_data, response_model=ExtractDataResponse)
    
    def markdown_converter(
            self, 
            link: Union[str, bytes, Any], 
            resource_type : MarkdownResourceType
        ) -> MarkdownCoverterResponse:
        """
        Convert structured data from a website or file as Markdown.
        
        Endpoint: POST /v2/markdown-converter/
        Note: This endpoint expects multipart/form-data.
        """
        link = self._get_resource("python_sdk_markdown_collection_id",link)
        form_data = {
            "link": link,
            "resource_type": resource_type
        }
        return self._request("POST", "/v2/markdown-converter/", data=form_data, response_model=MarkdownCoverterResponse)
    
    def transcript(
            self, 
            link: URL, 
            resource_type : TranscribeResourceType
        ) -> TrascribeResponse:
        """
        Extract Trascribe text from Youtube.
        
        Endpoint: POST /v2/transcript/
        Note: This endpoint expects multipart/form-data.
        """
        form_data = {
            "link": link,
            "resource_type": resource_type
        }
        return self._request("POST", "/v2/transcript/", data=form_data, response_model=TrascribeResponse)


# Example usage:
if __name__ == "__main__":
    # Replace 'your_api_key_here' with your actual API key
    # client = WetrocloudClient(api_key="your_api_key_here")   
    try:
        pass
        # Create a new collection
        # create_response = client.create_collection("sdk_unique_id_3")
        # print("Collection Created: %s", create_response)
        # collection_id = create_response.collection_iD
        # print(create_response.collection_id)
        
        # if collection_id:
        #     # collection_id = "sdk_unique_id_1"
        # #     # Insert a resource into the collection
        #     insert_response = client.insert_resource(
        #         collection_id=collection_id,
        #         resource="https://medium.com/@wetrocloud/rag-vs-fine-tuning-which-one-should-you-use-for-your-ai-workflow-5a71fc56ed77",
        #         type="web"
        #     )
        #     resource_id = insert_response.resource_id
        #     print("Resource Inserted: %s", insert_response)
            
        # #     # Query the collection for an answer
        #     query_response = client.query_collection(
        #         collection_id=collection_id,
        #         request_query="What are the key points of the article?"
        #     )
        #     print("Query Response: %s", query_response)
            
        #     # Chat with the collection
            
        #     chat_history = [
        #         {"role": "user", "content": "What is this collection about?"},
        #         {"role": "system", "content": "It stores web articles and documents."}
        #     ]
        #     chat_response = client.chat_with_collection(
        #         collection_id=collection_id,
        #         message="Can you summarize the content?",
        #         chat_history=chat_history
        #     )
        #     print("Chat Response: %s", chat_response)
            
        #     # Remove a resource (example resource_id; replace with a real one)
        #     remove_response = client.remove_resource(
        #         collection_id=collection_id,
        #         resource_id=resource_id
        #     )
        #     print("Resource Removal: %s", remove_response)
            
        #     # Delete the collection when done
        #     delete_response = client.delete_collection(collection_id)
        #     print("Collection Deletion: %s", delete_response)

        # # Data categorization example:
        # category_response = client.categorize_data(
        #     resource="match review: John Cena vs. The Rock.",
        #     type="text",
        #     json_schema="{\"label\":\"string\"}",
        #     categories=["football", "coding", "entertainment", "basketball", "wrestling", "information"]
        # )
        # print(category_response.response)
        
        # # Text generation example:
        # messages = [{"role": "user", "content": "what is a large language model?"}]
        # text_gen_response = client.generate_text(messages, model="llama-3.3-70b")
        # print(text_gen_response.response)
        
        # # Image to text (OCR) example:
        # image_response = client.image_to_text(
        #     image_url="https://encrypted-tbn0.gstatic.com/images?q=tbn:ANd9GcTQBQcwHfud1w3RN25Wgys6Btt_Y-4mPrD2kg&s",
        #     request_query="What animal is this?"
        # )
        # print(image_response.response)
        
        # # Data extraction example:
        # extraction_response = client.extract_data(
        #     website="https://www.forbes.com/real-time-billionaires/#7583ee253d78",
        #     json_schema='[{"name":"<name of rich man>", "networth":"<amount worth>"}]'
        # )
        # print(extraction_response.response)     
    except WetrocloudAPIError as e:
        print("An error occurred: %s", e)



from typing import Any, Dict, List, Union
from typing_extensions import Literal, TypedDict, TypeAlias
# Custom type definitions
ChatModel: TypeAlias = Literal[
    "chatgpt-4o-latest", 
    "claude-3-5-haiku-20241022", 
    "claude-3-5-sonnet-20240620",
    "claude-3-5-sonnet-20241022", 
    "claude-3-7-sonnet-20250219", 
    "claude-3-haiku-20240307",
    "claude-3-opus-20240229", 
    "claude-3-sonnet-20240229", 
    "deepseek-r1-distill-llama-70b",
    "deepseek-r1-distill-llama-70b-specdec", 
    "deepseek-r1-distill-qwen-32b", 
    "gpt-3.5-turbo",
    "gpt-4", 
    "gpt-4-turbo", 
    "gpt-4-turbo-preview", 
    "gpt-4.5-preview", 
    "gpt-4o", 
    "gpt-4o-mini",
    "llama-3.1-8b", 
    "llama-3.1-8b-instant", 
    "llama-3.2-1b-preview", 
    "llama-3.2-3b-preview",
    "llama-3.2-11b-vision-preview", 
    "llama-3.2-90b-vision-preview", 
    "llama-3.3-70b",
    "llama-3.3-70b-specdec", 
    "llama-3.3-70b-versatile", 
    "llama3-70b-8192", 
    "llama3-8b-8192",
    "llama-guard-3-8b", 
    "mixtral-8x7b-32768", 
    "o1", 
    "o1-mini", 
    "o1-preview", 
    "o3-mini",
    "qwen-2.5-32b", 
    "qwen-2.5-coder-32b"
]

ResourceType: TypeAlias = Literal["text", "web", "file", "json", "youtube"]

MarkdownResourceType: TypeAlias = Literal["web", "file", "image"]

TranscribeResourceType: TypeAlias = Literal["youtube"]

InsertStatus: TypeAlias = Literal["PENDING", "COMPLETED", "FAILED"]

class Message(TypedDict):
    role: Literal["system", "user", "assistant"]
    content: str

class CollectionItem(TypedDict):
    collection_id: Union[None,str]
    created_at: Union[None,str]

ChatHistory: TypeAlias = List[Message]

JSONSchema: TypeAlias = Union[None, bool, int, float, str, List[Any], Dict[str, Any]]
Categories: TypeAlias = List[str]

CollectionID: TypeAlias = str
ResourceID: TypeAlias = str
URL: TypeAlias = str




from wetro.api_client import WetrocloudClient
from typing import List,Optional,Union, Any
from wetro.custom_types import ChatModel, ResourceType, Categories, JSONSchema, Message, URL, MarkdownResourceType, TranscribeResourceType
from wetro.custom_response import CategorizeResponse, GenerateTextResponse, ImageToTextResponse, ExtractDataResponse, MarkdownCoverterResponse, TrascribeResponse

class WetroTools:
    """
    Client for additional tools provided by Wetrocloud Outside RAG.
    
    Usage:
        tools_client = WetroTools(api_key="your_api_key")
        res = tools_client.categorize(
            resource="match review: John Cena vs. The Rock.",
            type="text",
            json_schema='{"label": "string"}',
            categories=["wrestling", "entertainment"]
        )
        res = tools_client.generate_text(
            messages=[{"role": "user", "content": "What is a large language model?"}],
            model="gpt-4"
        )
        res = tools_client.image_to_text(
            image_url="https://example.com/image.jpg",
            request_query="What animal is this?"
        )
        res = tools_client.extract(
            website="https://www.forbes.com/real-time-billionaires/",
            json_schema='[{"name": "<name>", "networth": "<networth>"}]'
        )
    """
    def __init__(
        self,
        api_key: Optional[str] = None,
        base_url: str = "https://api.wetrocloud.com",
        timeout: int = 30,
        client: Optional[WetrocloudClient] = None
    ):
        if client is None:
            if api_key is None:
                raise ValueError("Must provide either an API key or a client instance")
            self._client = WetrocloudClient(api_key, base_url, timeout)
        else:
            self._client = client

    def categorize(
            self, 
            resource: str, 
            type: ResourceType, 
            json_schema: JSONSchema, 
            categories: Categories,
            prompt: str
        ) -> CategorizeResponse:
        """
        Categories your resource.
        """
        return self._client.categorize_data(resource, type, json_schema, categories, prompt)

    def generate_text(
            self, 
            messages: List[Message], 
            model: ChatModel
        ) -> GenerateTextResponse:
        """
        Generate Text-Response based on message
        """
        return self._client.generate_text(messages, model)

    def image_to_text(
            self, 
            image_url: URL, 
            request_query: str
        ) -> ImageToTextResponse:
        """
        Generate Text-Response based on image
        """
        return self._client.image_to_text(image_url, request_query)

    def extract(
            self, 
            website: URL, 
            json_schema: JSONSchema
        ) -> ExtractDataResponse:
        """
        Extract Data from Websites
        """
        return self._client.extract_data(website, json_schema)
    
    def markdown_converter(
            self, 
            link: Union[str, bytes, Any], 
            resource_type : MarkdownResourceType
        ) -> MarkdownCoverterResponse:
        """
        Convert Data to Markdown
        """
        return self._client.markdown_converter(link, resource_type )
    
    def transcript(
            self, 
            link: URL, 
            resource_type : TranscribeResourceType
        ) -> TrascribeResponse:
        """
        Trascribe Youtube Videos
        """
        return self._client.transcript(link, resource_type )
    
